@@ -1,5 +1,6 @@
 const Applet = imports.ui.applet;
 const GLib = imports.gi.GLib;
+const Gio = imports.gi.Gio;
 const GObject = imports.gi.GObject;
 const Lang = imports.lang;
 const NM = imports.gi.NM;
@@ -11,6 +12,7 @@ const PopupMenu = imports.ui.popupMenu;
 const MessageTray = imports.ui.messageTray;
 const ModemManager = imports.misc.modemManager;
 const Util = imports.misc.util;
+const Settings = imports.ui.settings;
 
 const DEFAULT_PERIODIC_UPDATE_FREQUENCY_SECONDS = 10;
 const FAST_PERIODIC_UPDATE_FREQUENCY_SECONDS = 2;
@@ -42,6 +44,23 @@ const NM80211ApSecurityFlags = NM['80211ApSecurityFlags'];
 // (the remaining are placed into More...)
 const NUM_VISIBLE_NETWORKS = 5;
 
+var NMIface = '\
+<node> \
+  <interface name="org.freedesktop.NetworkManager"> \
+    <method name="CheckConnectivity"> \
+      <arg type="u" name="connectivity" direction="out"/> \
+    </method> \
+  </interface> \
+</node>';
+
+var NMProxy = Gio.DBusProxy.makeProxyWrapper(NMIface);
+function NMDBus(initCallback, cancellable) {
+    return new NMProxy(Gio.DBus.system,
+                       'org.freedesktop.NetworkManager',
+                       '/org/freedesktop/NetworkManager',
+                       initCallback, cancellable);
+}
+
 // shared between NMNetworkMenuItem and NMDeviceWWAN
 function signalToIcon(value) {
     if (value > 80)
@@ -257,7 +276,7 @@ NMWirelessSectionTitleMenuItem.prototype = {
 
     activate: function(event) {
         PopupMenu.PopupSwitchMenuItem.prototype.activate.call(this, event);
-		log(this._setEnabledFunc);
+
         this._client[this._setEnabledFunc](this._switch.state);
 
         if (!this._device) {
@@ -478,7 +497,7 @@ NMDevice.prototype = {
     },
 
     setEnabled: function(enabled) {
-        // do nothing by default, we want to keep the conneciton list visible
+        // do nothing by default, we want to keep the connection list visible
         // in the majority of cases (wired, wwan, vpn)
     },
 
@@ -623,7 +642,7 @@ NMDevice.prototype = {
         if (active) {
             title = active._name;
         } else {
-            /* TRANSLATORS: this is the indication that a connection for another logged in user is active,
+            /* Translators: this is the indication that a connection for another logged in user is active,
                and we cannot access its settings (including the name) */
             title = _("Connected (private)");
         }
@@ -1532,7 +1551,7 @@ NMDeviceWireless.prototype = {
         let name;
         let ssid = NM.utils_ssid_to_utf8(apObj.ssid.get_data());
         if (ssid) {
-            /* TRANSLATORS: this the automatic wireless connection name (including the network name) */
+            /* Translators: this the automatic wireless connection name (including the network name) */
             name = _("Auto %s").format(ssid);
         } else
             name = _("Auto wireless");
@@ -1661,8 +1680,8 @@ CinnamonNetworkApplet.prototype = {
 
         try {
             this.metadata = metadata;
-            Main.systrayManager.registerRole("network", metadata.uuid);
-            Main.systrayManager.registerRole("nm-applet", metadata.uuid);
+            Main.systrayManager.registerTrayIconReplacement("network", metadata.uuid);
+            Main.systrayManager.registerTrayIconReplacement("nm-applet", metadata.uuid);
 
             this.menuManager = new PopupMenu.PopupMenuManager(this);
             this.menu = new Applet.AppletPopupMenu(this, orientation);
@@ -1671,6 +1690,10 @@ CinnamonNetworkApplet.prototype = {
             this._currentIconName = undefined;
             this._setIcon('network-offline');
 
+            this.settings = new Settings.AppletSettings(this, metadata.uuid, this.instance_id);
+            this.settings.bind("keyOpen", "keyOpen", this._setKeybinding);
+            this._setKeybinding();
+
             NM.Client.new_async(null, Lang.bind(this, this._clientGot));
         }
         catch (e) {
@@ -1678,9 +1701,22 @@ CinnamonNetworkApplet.prototype = {
         }
     },
 
+    _setKeybinding() {
+        Main.keybindingManager.addHotKey("network-open-" + this.instance_id, this.keyOpen, Lang.bind(this, this._openMenu));
+    },
+
     _clientGot: function(obj, result) {
         try {
             this._client = NM.Client.new_finish(result);
+            this._v1_28_0 = Util.version_exceeds(this._client.get_version(), "1.28.0");
+
+            this._nm_proxy = null;
+
+            if (!this._v1_28_0) {
+                new NMDBus(Lang.bind(this, function(proxy, error) {
+                    this._nm_proxy = proxy;
+                }));
+            }
 
             this._statusSection = new PopupMenu.PopupMenuSection();
             this._statusItem = new PopupMenu.PopupMenuItem('', { style_class: 'popup-inactive-menu-item', reactive: false });
@@ -1748,7 +1784,9 @@ CinnamonNetworkApplet.prototype = {
             this.menu.connect("open-state-changed", Lang.bind(this, this._onMenuOpenStateChanged));
 
             this._periodicTimeoutId = 0;
-            this._updateFrequencySeconds = DEFAULT_PERIODIC_UPDATE_FREQUENCY_SECONDS;
+            this._updateFrequencySeconds = 0;
+            this._checkingConnectivity = false;
+            this._lastConnectivityState = NM.ConnectivityState.UNKNOWN;
 
             this._activeConnections = [ ];
             this._connections = [ ];
@@ -1807,6 +1845,10 @@ CinnamonNetworkApplet.prototype = {
         this.menu.toggle();
     },
 
+    _openMenu: function () {
+        this.menu.toggle();
+    },
+
     _ensureSource: function() {
         if (!this._source) {
             this._source = new NMMessageTraySource();
@@ -2047,15 +2089,16 @@ CinnamonNetworkApplet.prototype = {
                 } else
                     a._primaryDevice = this._devices.vpn.device;
 
-                if (a._primaryDevice)
-                    a._primaryDevice.setActiveConnection(a);
-
                 if (a.state == NM.ActiveConnectionState.ACTIVATED &&
                     a._primaryDevice && a._primaryDevice._notification) {
                         a._primaryDevice._notification.destroy();
                         a._primaryDevice._notification = null;
                 }
             }
+
+            if (a._primaryDevice) {
+                a._primaryDevice.setActiveConnection(a);
+            }
         }
 
         this._mainConnection = activated || activating || default_ip4 || default_ip6 || null;
@@ -2225,6 +2268,9 @@ CinnamonNetworkApplet.prototype = {
                 }
             } else {
                 let dev;
+                let limited_conn = false;
+                // let limited_conn = this._correctStateForTunnel(this._client.get_connectivity()) !== NM.ConnectivityState.FULL;
+
                 switch (mc._section) {
                 case NMConnectionCategory.WIRELESS:
                     dev = mc._primaryDevice;
@@ -2239,16 +2285,27 @@ CinnamonNetworkApplet.prototype = {
                             this._setIcon('network-wireless-connected');
                             this.set_applet_tooltip(_("Connected to the wireless network"));
                         } else {
-                            this._setIcon('network-wireless-signal-' + signalToIcon(ap.strength));
-                            this.set_applet_tooltip(_("Wireless connection") + ": " + ssidToLabel(ap.get_ssid()) + " ("+ ap.strength +"%)");
+                            if (limited_conn) {
+                                this._setIcon('network-wireless-no-route');
+                                this.set_applet_tooltip(_("Wireless connection") + ": " + ssidToLabel(ap.get_ssid()) + " " + _("(Limited connectivity)"));
+                            } else {
+                                this._setIcon('network-wireless-signal-' + signalToIcon(ap.strength));
+                                this.set_applet_tooltip(_("Wireless connection") + ": " + ssidToLabel(ap.get_ssid()) + " ("+ ap.strength +"%)");
+                            }
                         }
                     } else {
                         log('Active connection with no primary device?');
                     }
                     break;
                 case NMConnectionCategory.WIRED:
-                    this._setIcon('network-wired');
-                    this.set_applet_tooltip(_("Connected to the wired network"));
+                    if (limited_conn) {
+                        this._setIcon('network-wired-no-route');
+                        this.set_applet_tooltip(_("Connected to the wired network") + " " + _("(Limited connectivity)"));
+
+                    } else {
+                        this._setIcon('network-wired');
+                        this.set_applet_tooltip(_("Connected to the wired network"));
+                    }
                     break;
                 case NMConnectionCategory.WWAN:
                     dev = mc._primaryDevice;
@@ -2263,10 +2320,18 @@ CinnamonNetworkApplet.prototype = {
                         break;
                     }
 
-                    this._setIcon('network-cellular-signal-' + signalToIcon(dev.mobileDevice.signal_quality));
-                    this.set_applet_tooltip(_("Connected to the cellular network"));
+                    if (limited_conn) {
+                        this._setIcon('network-cellular-no-route');
+                        this.set_applet_tooltip(_("Connected to the cellular network") + " " + _("(Limited connectivity)"));
+                    } else {
+                        this._setIcon('network-cellular-signal-' + signalToIcon(dev.mobileDevice.signal_quality));
+                        this.set_applet_tooltip(_("Connected to the cellular network"));
+                    }
+
                     break;
                 case NMConnectionCategory.VPN:
+                    // Should we indicate limited connectivity for VPNs like we do above? What if the connection is to
+                    // a local machine? Need to test.
                     this._setIcon('network-vpn');
                     this.set_applet_tooltip(_("Connected to the VPN"));
                     break;
@@ -2292,6 +2357,25 @@ CinnamonNetworkApplet.prototype = {
 
     _periodicUpdateIcon: function() {
         let new_delay = this._updateIcon();
+        this._lastConnectivityState = this._client.get_connectivity();
+
+        // TODO: This can be unreliable with multiple interfaces, and libnm
+        // can be flaky with its connectivity state - on one machine of mine,
+        // I observed regular switching between full and limited connectivity,
+        // when no noticeable change in my actual ability to reach the internet.
+        // We might re-implement to be independent of libnm. For now just disable it.
+        //
+        // if (!this._checkingConnectivity) {
+        //     // https://gitlab.freedesktop.org/NetworkManager/NetworkManager/-/issues/476
+        //     if (this._v1_28_0) {
+        //         this._checkingConnectivity = true;
+        //         this._client.check_connectivity_async(null, Lang.bind(this, this._connectivityCheckCallback));
+        //     } else
+        //     if (this._nm_proxy != null) {
+        //         this._checkingConnectivity = true;
+        //         this._nm_proxy.CheckConnectivityRemote(Lang.bind(this, this._proxyConnectivityCheckCallback));
+        //     }
+        // }
 
         if (this._updateFrequencySeconds != new_delay) {
             this._restartPeriodicUpdateTimer(new_delay);
@@ -2302,6 +2386,52 @@ CinnamonNetworkApplet.prototype = {
         return GLib.SOURCE_REMOVE;
     },
 
+    _correctStateForTunnel(state) {
+        if (state !== NM.ConnectivityState.LIMITED) {
+            return state;
+        }
+
+        // if our primary connection state is 'limited', check if there's a tunnel
+        // connection active, and assume 'full' if so - this prevents reporting limited
+        // connectivity when something like PIA vpn is active. This could end up reporting
+        // a full connection when there isn't one, depending on what the tunnel is for,
+        // but NM doesn't seem to provide any way to discern a relationship between a virtual
+        // device (tunnel) and the physical device in use. The physical device is always returned
+        // as the primary.
+
+        if (this._activeConnections.some(con => con.get_connection_type() === "tun")) {
+            return NM.ConnectivityState.FULL;
+        }
+    },
+
+    _proxyConnectivityCheckCallback(new_state) {
+        let state = this._correctStateForTunnel(new_state[0]);
+
+        if (state !== this._lastConnectivityState) {
+            this._updateIcon();
+        }
+
+        this._checkingConnectivity = false;
+    },
+
+    _connectivityCheckCallback(source, result) {
+        let new_state = NM.ConnectivityState.UNKNOWN;
+
+        try {
+            new_state = source.check_connectivity_finish(result);
+        } catch (e) {
+            log(e);
+        }
+
+        let state = this._correctStateForTunnel(new_state);
+
+        if (state !== this._lastConnectivityState) {
+            this._updateIcon();
+        }
+
+        this._checkingConnectivity = false;
+    },
+
     _restartPeriodicUpdateTimer: function(new_delay) {
         if (this._periodicTimeoutId > 0) {
             Mainloop.source_remove(this._periodicTimeoutId);
@@ -2340,8 +2470,8 @@ CinnamonNetworkApplet.prototype = {
     },
 
     on_applet_removed_from_panel: function() {
-        Main.systrayManager.unregisterRole("network", this.metadata.uuid);
-        Main.systrayManager.unregisterRole("nm-applet", this.metadata.uuid);
+        Main.systrayManager.unregisterTrayIconReplacement(this.metadata.uuid);
+        Main.keybindingManager.removeHotKey("network-open-" + this.instance_id);
         if (this._periodicTimeoutId){
             Mainloop.source_remove(this._periodicTimeoutId);
         }
