@@ -10,7 +10,7 @@ const Applet = imports.ui.applet;
 const Extension = imports.ui.extension;
 const ModalDialog = imports.ui.modalDialog;
 const {getModuleByIndex} = imports.misc.fileUtils;
-const {queryCollection, findIndex} = imports.misc.util;
+const {queryCollection} = imports.misc.util;
 const Gettext = imports.gettext;
 const Panel = imports.ui.panel;
 
@@ -22,16 +22,12 @@ var appletMeta;
 var appletObj = [];
 var appletsLoaded = false;
 
-// An applet can assume a role
-// Instead of hardcoding looking for a particular applet,
-// We let applets announce that they can fill a particular
-// role, using the 'role' metadata entry.
-// For now, just notifications, but could be expanded.
-// question - should multiple applets be able to fill
-// the same role?
+// FIXME: This role stuff is checked in extension.js, why not move checks from here to there?
 var Roles = {
     NOTIFICATIONS: 'notifications',
-    PANEL_LAUNCHER: 'panellauncher'
+    PANEL_LAUNCHER: 'panellauncher',
+    WINDOW_ATTENTION_HANDLER: 'windowattentionhandler',
+    WINDOW_LIST: 'windowlist'
 };
 
 var rawDefinitions;
@@ -283,7 +279,7 @@ function onEnabledAppletsChanged() {
         if (unChangedApplets.indexOf(oldDefinitions[i].applet_id) === -1) {
             removedApplets.push({changed: false, definition: oldDefinitions[i]});
         } else {
-            let removedIndex = findIndex(removedApplets, function(item) {
+            const removedIndex = removedApplets.findIndex( item => {
                 return item.definition.applet_id === oldDefinitions[i].applet_id;
             });
             if (removedIndex === -1) continue;
@@ -350,17 +346,20 @@ function removeAppletFromPanels(appletDefinition, deleteConfig, changed = false)
 }
 
 function _removeAppletConfigFile(uuid, instanceId) {
-    let config_path = (GLib.get_home_dir() + "/" +
-                               ".cinnamon" + "/" +
-                                 "configs" + "/" +
-                                      uuid + "/" +
-                                instanceId + ".json");
-    let file = Gio.File.new_for_path(config_path);
-    if (file.query_exists(null)) {
-        try {
-            file.delete(null);
-        } catch (e) {
-            global.logError("Problem removing applet config file during cleanup.  UUID is " + uuid + " and filename is " + config_path);
+    let config_paths = [
+        [GLib.get_home_dir(), ".cinnamon", "configs", uuid, instanceId + ".json"].join("/"),
+        [GLib.get_user_config_dir(), "cinnamon", "spices", uuid, instanceId + ".json"].join("/")
+    ];
+
+    for (let i = 0; i < config_paths.length; i++) {
+        const config_path = config_paths[i];
+        let file = Gio.File.new_for_path(config_path);
+        if (file.query_exists(null)) {
+            try {
+                file.delete(null);
+            } catch (e) {
+                global.logError("Problem removing applet config file during cleanup.  UUID is " + uuid + " and filename is " + config_path);
+            }
         }
     }
 }
@@ -417,7 +416,7 @@ function addAppletToPanels(extension, appletDefinition, panel = null, user_actio
 
         return true;
     } catch (e) {
-        extension.unlockRole();
+        extension.unlockRoles();
         Extension.logError('Failed to load applet: ' + appletDefinition.uuid + "/" + appletDefinition.applet_id, extension.uuid, e);
         return false;
     }
